from typing import Protocol

from src.mybootstrap_core_itskovichanton.config import ConfigService


class Application(Protocol):

    def __init__(self, config_service: ConfigService):
        self.config_service = config_service

    async def async_run(self):
        """Run in async mode"""

    def run(self):
        """Run sync"""



import smtplib
import ssl
from dataclasses import dataclass
from email.mime.multipart import MIMEMultipart
from email.mime.text import MIMEText
from typing import Protocol

from dacite import from_dict

from src.mybootstrap_core_itskovichanton.config import ConfigService


@dataclass
class EmailConfig:
    username: str
    password: str
    host: str
    address: str
    port: int
    encoding: str = "utf-8"


@dataclass
class Params:
    toEmail: list[str]
    senderEmail: str
    subject: str
    content_plain: str = ""
    content_html: str = ""


class EmailService(Protocol):

    def send(self, a: Params):
        """Send email"""


class EmailServiceImpl(EmailService):

    def __init__(self, config_service: ConfigService):
        email_settings = config_service.config.settings["email"]
        self.config = from_dict(data_class=EmailConfig, data=email_settings)

    def send(self, a: Params):
        if self.config is None and not a.content_plain and not a.content_html:
            return

        context = ssl.SSLContext(ssl.PROTOCOL_TLSv1_2)
        context.set_ciphers('DEFAULT@SECLEVEL=1')
        server = smtplib.SMTP_SSL(self.config.host, self.config.port, context=context)
        server.command_encoding = self.config.encoding
        server.login(self.config.username, self.config.password)
        m = MIMEMultipart()
        m["From"] = a.senderEmail,
        m["To"] = ', '.join(a.toEmail)
        m["Subject"] = a.subject
        if a.content_plain:
            m.attach(MIMEText(a.content_plain, "plain/text", _charset=self.config.encoding))
        elif not a.content_html:
            m.attach(MIMEText(a.content_html, "html", _charset=self.config.encoding))

        context = ssl.SSLContext(ssl.PROTOCOL_TLSv1_2)
        context.set_ciphers('DEFAULT@SECLEVEL=1')
        return server.sendmail(a.senderEmail, m["To"], m.as_string())



from opyoid import Module, Injector

from src.mybootstrap_core_itskovichanton.app import Application
from src.mybootstrap_core_itskovichanton.di import CoreModule
from tests.app import TestCoreApp


class TestCoreModule(Module):
    def configure(self) -> None:
        self.install(CoreModule)
        self.bind(Application, to_class=TestCoreApp)


injector = Injector([TestCoreModule])



import os
from dataclasses import dataclass
from typing import Protocol, Optional

import yaml
from dacite import from_dict


class ConfigService(Protocol):

    def __init__(self):
        self.config = None

    def dir(self) -> str:
        """App directory"""

    def app_name(self) -> str:
        """A Flyer can fly"""


@dataclass
class App:
    name: str
    version: str

    def full_name(self):
        return f"{self.name}-{self.version}"


@dataclass
class Config:
    app: App
    settings: Optional[dict]
    profile: Optional[str]

    def full_name(self):
        return f"{self.app.full_name()}-[{self.profile}]"

    def is_prod(self):
        return self.profile == "prod"


class ConfigLoaderService(Protocol):
    def load(self) -> Config:
        """Loads config instance"""


class YamlConfigLoaderService(ConfigLoaderService):

    def __init__(self, filename: str, profile: str):
        self.filename = filename
        self.profile = profile

    def load(self) -> Config:
        with open(self.filename) as f:
            settings: dict = yaml.load(f, Loader=yaml.FullLoader)
            profile_settings = settings[self.profile]
            for k, v in profile_settings.items():
                settings[k] = v
            r = from_dict(data_class=Config, data=settings)
            settings.pop(self.profile)
            r.settings = settings
            r.profile = self.profile

        return r


class ConfigServiceImpl(ConfigService):

    def __init__(self, config_loader: ConfigLoaderService):
        super().__init__()
        self.config = config_loader.load()
        self.dir()

    def app_name(self) -> str:
        return self.config.full_name()

    def dir(self) -> str:
        r = os.path.join(self.config.app.full_name(), "work", self.config.profile)
        os.makedirs(r, exist_ok=True)
        return r

    # def on_dir(self, *args) -> str:
    #     r = os.path.join(self.dir(), list(args))
    #     os.makedirs(r, exist_ok=True)
    #     return r



import asyncio

from src.mybootstrap_core_itskovichanton.app import Application
from tests.di import injector


async def main() -> None:
    app = injector.inject(Application)
    await app.async_run()
    # app.run()


if __name__ == '__main__':
    # main()
    loop = asyncio.new_event_loop()
    asyncio.set_event_loop(loop)
    try:
        asyncio.run(main())
    except KeyboardInterrupt:
        pass



import functools
import logging
import traceback
from dataclasses import dataclass
from typing import Protocol

from src.mybootstrap_core_itskovichanton.config import ConfigService
from src.mybootstrap_core_itskovichanton.email_service import EmailService, Params
from src.mybootstrap_core_itskovichanton.fr import FRService, Post
from src.mybootstrap_core_itskovichanton.utils import trim_string


@dataclass
class Alert:
    message: str = None
    subject: str = None
    byEmail: bool = True
    byFR: bool = True
    level: int = 1
    send: bool = True


class AlertService(Protocol):

    async def send(self, a: Alert):
        """Send alert"""

    async def handle(self, e: BaseException, alert: Alert = Alert()):
        """Send alert about exception"""


alert_service: AlertService


class AlertServiceImpl(AlertService):

    def __init__(self, config_service: ConfigService, fr_service: FRService, email_service: EmailService):
        self.config_service = config_service
        self.fr_service = fr_service
        self.email_service = email_service
        global alert_service
        alert_service = self
        self.emails = self.config_service.config.settings["alerts"]["emails"]
        self.from_email = self.config_service.config.settings["email"]["from"]

    async def send(self, a: Alert):
        if not a.send:
            return

        if not a.subject:
            a.subject = "NO TOPIC"

        a.subject = f"{self.config_service.app_name()} - [{a.subject}]"

        if a.byFR:
            await self.send_by_fr(a)

        if a.byEmail:
            pass
            # self.send_by_email(a)

    async def send_by_fr(self, a):
        await self.fr_service.send(Post(project=a.subject, level=a.level, msg=trim_string(a.message, 4000)))

    def send_by_email(self, a):
        self.email_service.send(
            Params(subject=a.subject, toEmail=self.emails, senderEmail=self.from_email, content_plain=a.message))

    async def handle(self, e: BaseException, alert: Alert = Alert()):
        logging.exception(e)
        alert.message = traceback.format_exc()
        alert.subject = "Exception"
        await self.send(alert)


def alert_on_fail(method, alert: Alert = Alert()):
    @functools.wraps(method)
    async def _impl(self, *method_args, **method_kwargs):
        try:
            return await method(self, *method_args, **method_kwargs)
        except BaseException as e:
            await alert_service.handle(e, alert)

    return _impl



from opyoid import Module, Injector

from src.mybootstrap_core_itskovichanton.alerts import AlertService, AlertServiceImpl
from src.mybootstrap_core_itskovichanton.config import YamlConfigLoaderService, ConfigLoaderService, ConfigService, ConfigServiceImpl
from src.mybootstrap_core_itskovichanton.email_service import EmailServiceImpl, EmailService
from src.mybootstrap_core_itskovichanton.fr import FRService, FRServiceImpl


class CoreModule(Module):
    def configure(self) -> None:
        self.bind(ConfigLoaderService, to_instance=YamlConfigLoaderService("config.yml", "dev"))
        self.bind(ConfigService, to_class=ConfigServiceImpl)
        self.bind(FRService, to_class=FRServiceImpl)
        self.bind(EmailService, to_class=EmailServiceImpl)
        self.bind(AlertService, to_class=AlertServiceImpl)
        # self.bind(Application, to_class=TestCoreApplication)
        self.bind(Injector, to_instance=Injector([self]))


injector = Injector([CoreModule])




from src.mybootstrap_core_itskovichanton.alerts import AlertService, alert_on_fail, Alert
from src.mybootstrap_core_itskovichanton.app import Application
from src.mybootstrap_core_itskovichanton.config import ConfigService


class TestCoreApp(Application):

    def __init__(self, config_service: ConfigService, alert_service: AlertService):
        super().__init__(config_service)
        self.alert_service = alert_service

    def run(self):
        print(self.config_service.app_name())

    async def async_run(self):
        print(self.config_service.app_name())
        # self.http_controller.start()
        # print(await self.do_stuff_with_errors())
        await self.alert_service.send(Alert(level=1, message="Test", subject="Error happened", byEmail=False))
        # uvicorn.run("main:app", reload=True, workers=4)

    # @wrapper()
    def do_stuff_with_errors1(self):
        return "Hello"

    @alert_on_fail
    async def do_stuff_with_errors(self):
        return f"{1 / 0}"



from contextlib import suppress
from dataclasses import dataclass
from typing import Protocol

import httpx
from dacite import from_dict

from src.mybootstrap_core_itskovichanton.config import ConfigService


@dataclass
class FRConfig:
    url: str
    developer_id: int


@dataclass
class Post:
    project: str
    msg: str
    level: int


class FRService(Protocol):

    async def send(self, a: Post):
        """Send post to fr"""


class FRServiceImpl(FRService):

    def __init__(self, config_service: ConfigService):
        fr_settings = config_service.config.settings["fr"]
        self.config = from_dict(data_class=FRConfig, data=fr_settings)
        self.http_client = httpx.AsyncClient()

    async def send(self, a: Post):
        if self.config is None:
            return
        with suppress(BaseException):
            await self.http_client.post(self.config.url + "/postMsg",
                                        data={'msg': a.msg, 'project': a.project, 'level': a.level})
